# Stock Index MCVaR Core.R
fSIFMCVaRCore = function(FuturesInputData, InitialValuations, Variable){
  Horizon = SimulationInputData$Horizon
  NumberOfRandomVariables <- 3.0 # Stock, LCC, SCC
  ConfidenceLevel <- SimulationInputData$ConfidenceLevel
  # Extract Name, Units, Price, Mean, Standard Deviation, and Correlation inputs
  Name = SimulationInputData$Name
  Units = inputUnits
  Price = inputStockIndex1 # Just one stock for now
  Mean = as.numeric(c(1:NumberOfRandomVariables))
  Mean[1] <- SimulationInputData$StockMean/100.0
  Mean[2] <- SimulationInputData$LCCMean/100.0
  Mean[3] <- SimulationInputData$SCCMean/100.0
  StandardDeviation = as.numeric(c(1:NumberOfRandomVariables))
  StandardDeviation[1] <- SimulationInputData$StockSD/100.0
  StandardDeviation[2] <- SimulationInputData$LCCSD/100.0
  StandardDeviation[3] <- SimulationInputData$SCCSD/100.0
  Correlation = matrix(data = 0, nrow = NumberOfRandomVariables, ncol = NumberOfRandomVariables)
  Correlation[1,1] <- Correlation[2,2] <- Correlation[3,3] <- 1.0
  Correlation[1,2] <- Correlation[2,1] <- SimulationInputData$CorStockLCC
  Correlation[1,3] <- Correlation[3,1] <- SimulationInputData$CorStockSCC
  Correlation[2,3] <- Correlation[3,2] <- SimulationInputData$CorLCCSCC
  Covariance = matrix(nrow = NumberOfRandomVariables, ncol = NumberOfRandomVariables)
  Covariance[1,1] <- StandardDeviation[1]^2
  Covariance[2,2] <- StandardDeviation[2]^2 
  Covariance[3,3] <- StandardDeviation[3]^2 
  Covariance[1,2] <- Covariance[2,1] <- StandardDeviation[1] * StandardDeviation[2] * Correlation[1,2]
  Covariance[1,3] <- Covariance[3,1] <- StandardDeviation[1] * StandardDeviation[3] * Correlation[1,3]
  Covariance[2,3] <- Covariance[3,2] <- StandardDeviation[2] * StandardDeviation[3] * Correlation[2,3]
#
# Source all required functions
#
  source("Stock Index Futures MCVaR Functions.R")
#
# VaR analysis of European-style options
#
  IV <- fInitialValues(FuturesInputData, InitialValuations) # NOTE: This function will change with option models
#
# Set up simulations
#
# Test normal distribution (Normal with mean = Mean and
#   standard deviation = StandardDeviation)
# Sample (s)
  NSim <- SimulationInputData$NumberOfSimulations
# If simulated covariance is not positive definite, then adjust  
  if(!is.positive.definite(Covariance)){ # Is Dmat not positive definite?
    pd_D_mat <- nearPD(Covariance)  # Find nearby positive definite solution
    Covariance = as.matrix(pd_D_mat$mat)
    Errorpd_D_mat <- pd_D_mat
  }
  s = mvrnorm(n = NSim, Mean, Covariance, tol = 1e-6, empirical = FALSE, EISPACK = FALSE)
  sMean = as.numeric(c(1:NumberOfRandomVariables))
  sStandardDeviation = as.numeric(c(1:NumberOfRandomVariables))
  sCorrelation = matrix(nrow = NumberOfRandomVariables, ncol = NumberOfRandomVariables)
  for(i in 1:NumberOfRandomVariables){
    sMean[i] = mean(s[,i])
    sStandardDeviation[i] = sd(s[,i])
    for(j in 1:i){
      sCorrelation[i, j] = cor(s[,i],s[,j])
      sCorrelation[j, i] = sCorrelation[i, j]
    }
  }
  TerminalValuations = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NSim))
  colnames(TerminalValuations)  <- seq(1:NSim)
  rownames(TerminalValuations)  <- c("LS", # Long stock index
    "LSFN1", "LSFN2", "LSFN3", # Long stock index futures (nearby 1, 2, and 3)
    "SSFN1", "SSFN2", "SSFN3", # Short stock index futures (nearby 1, 2, and 3)
    "LST1N1", "LST1N2", "LST1N3", # Long spread trade (long nearby, short 2nd nearby ...)
    "SST1N1", "SST1N2", "SST1N3", # Short spread trade (short nearby, long 2nd nearby ...)
    "LST2N1", "LST2N2", "LST2N3", # Long spread trade 2 (long nearby, short 3rd nearby ...)
    "SST2N1", "SST2N2", "SST2N3" # Short spread trade 2 (short nearby, long 3rd nearby ...)
  )
  Horizon <- SimulationInputData$Horizon
  tStockIndex <- FuturesInputData$StockIndex
  tLCC <- FuturesInputData$LevelCC
  tSCC <- FuturesInputData$SlopeCC
  tTimeToMaturity <- FuturesInputData$TimeToMaturity
  FuturesInputData$TimeToMaturity <- tTimeToMaturity - Horizon 
  for(i in 1:NSim){
    FuturesInputData$StockIndex <- tStockIndex*exp(s[i,1]*Horizon)  
    FuturesInputData$LevelCC <- tLCC + s[i,2]*Horizon
    FuturesInputData$SlopeCC <- tSCC + s[i,3]*Horizon
    tempTV <- fInitialValues(FuturesInputData, InitialValuations)
    TerminalValuations[,i] <- tempTV["InitialValue"]
  }
  FuturesInputData$TimeToMaturity <- tTimeToMaturity
  FuturesInputData$StockIndex <- tStockIndex
  FuturesInputData$LevelCC <- tLCC
  FuturesInputData$SlopeCC <- tSCC
#
# Compute VaR for each strategy
# 
  TerminalValue <- rep(-99, 19)
  Valuations <- cbind(IV, TerminalValue)
  VaR <- rep(-99, 19)
  Valuations <- cbind(Valuations, VaR)
  RVaR <- rep(-99, 19)
  Valuations <- cbind(Valuations, RVaR)
  for(i in 1:NumberOfStrategies){
    tVector <- as.matrix(TerminalValuations[i,])
    VaRTV <- as.numeric(quantile(tVector, 1-ConfidenceLevel/100))
    Valuations[i,3] <- as.numeric(VaRTV)
    Valuations[i,4] <- -(as.numeric(Valuations[i,3]) - as.numeric(Valuations[i,1])) 
    Valuations[i,5] <- (Valuations[i,4]/Valuations[i,2])*100.0
  }
  Valuations <- round(Valuations,2)  
# Record base case information  
  if(BASECASE){
    Valuations <- cbind(Strategy = rownames(Valuations), Valuations)
    rownames(Valuations) <- 1:nrow(Valuations)
    Tab <- "VaR Output"
    wb <- buildWorkbook(Valuations, sheetName = Tab, gridLines = TRUE)
# Input option data
    Tab <- "Futures Parameters"
    addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
    FuturesInputData <- t(FuturesInputData)
    FuturesInputData <- cbind(Strategy = rownames(FuturesInputData), FuturesInputData)
    rownames(FuturesInputData) <- 1:nrow(FuturesInputData)
    writeData(wb, Tab, FuturesInputData)
# Input simulation data
    Tab <- "Simulation Parameters"
    addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
    SimulationInputData <- as.data.frame(SimulationInputData)
    SimulationInputData <- t(SimulationInputData)
    SimulationInputData <- cbind(Parameters = rownames(SimulationInputData), SimulationInputData)
    rownames(SimulationInputData) <- 1:nrow(SimulationInputData)
    writeData(wb, Tab, SimulationInputData)
#
# Build data frame to contain both input simulation parameters as well as output
#
    dfMean <- as.data.frame(Mean)
    colnames(dfMean)  <- c("PopMean")
    dfSMean <- as.data.frame(sMean)
    colnames(dfSMean)  <- c("SamMean")
    dfStandardDeviation <- as.data.frame(StandardDeviation)
    colnames(dfStandardDeviation)  <- c("PopSD")
    dfSStandardDeviation <- as.data.frame(sStandardDeviation)
    colnames(dfSStandardDeviation)  <- c("SamSD")
    dfCorrelation <- as.data.frame(Correlation)
    colnames(dfCorrelation)  <- c("PopCor1", "PopCor2", "PopCor3")
    dfSCorrelation <- as.data.frame(sCorrelation)
    colnames(dfSCorrelation)  <- c("SamCor1", "SamCor2", "SamCor3")
    dfMean <- dfMean*100
    dfSMean <- dfSMean*100
    dfStandardDeviation <- dfStandardDeviation*100
    dfSStandardDeviation <- dfSStandardDeviation*100
    SimParameters <- cbind(dfMean, dfSMean, dfStandardDeviation, dfSStandardDeviation,
      dfCorrelation, dfSCorrelation)
    SimParameters <- round(SimParameters, 2)
    rownames(SimParameters)  <- c("Stock", "Level CC", "Slope CC")
    SimParameters <- cbind(Parameters = rownames(SimParameters), SimParameters)
    Tab <- "Simulation Statistics"
    addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
    writeData(wb, Tab, SimParameters)
    FilePath <- "Selected Output/"
    WorkBookName <- paste0(FilePath, 'SIF VaR and Valuations ', Variable, ' 1', '.xlsx')
    saveWorkbook(wb, WorkBookName, overwrite = TRUE)
  }
  return(Valuations)
}
