# Stock Index MCVaR Functions.R
#
source("FCAVM With SRM Functions.R")
#
# Portfolio risk management functions
#
# Set initial values for 19 positions
fInitialValues = function(FuturesInputData, InitialValuations){
  tTTM <- FuturesInputData$TimeToMaturity
  tIV <- InitialValuations
# Long stock index
  tIV["LS", "InitialValue"] <- FuturesInputData$Units * FuturesInputData$StockIndex
  tIV["LS", "Margin"] <- (FuturesInputData$ECC/100)*abs(tIV["LS", "InitialValue"])
# Long 1st nearby futures  
  Value <-  FVCA(FuturesInputData)
  tIV["LSFN1", "InitialValue"] <- FuturesInputData$Units * Value
  tIV["LSFN1", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LSFN1", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Long 2nd nearby futures
  FuturesInputData$TimeToMaturity <- tTTM + 0.25
  Value <- FVCA(FuturesInputData)
  tIV["LSFN2", "InitialValue"] <- FuturesInputData$Units * Value
  tIV["LSFN2", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LSFN2", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Long 3rd nearby futures
  FuturesInputData$TimeToMaturity <- tTTM + 0.25*2
  Value <- FVCA(FuturesInputData)
  tIV["LSFN3", "InitialValue"] <- FuturesInputData$Units * Value
  tIV["LSFN3", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LSFN3", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short 1st nearby futures
  tIV["SSFN1", "InitialValue"] <- -tIV["LSFN1", "InitialValue"]
  tIV["SSFN1", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SSFN1", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short 2nd nearby futures
  tIV["SSFN2", "InitialValue"] <- -tIV["LSFN2", "InitialValue"]
  tIV["SSFN2", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SSFN2", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short 3rd nearby futures
  tIV["SSFN3", "InitialValue"] <- -tIV["LSFN3", "InitialValue"]
  tIV["SSFN3", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SSFN3", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Long spread trade version 1: long 1st nearby, short 2nd nearby
  FuturesInputData$TimeToMaturity <- tTTM + 0.25*3
  Value <- FVCA(FuturesInputData)
  SSF4 <- -FuturesInputData$Units * Value
  tIV["LST1N1", "InitialValue"] <- tIV["LSFN1", "InitialValue"] +  tIV["SSFN2", "InitialValue"]
  tIV["LST1N1", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LST1N1", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Long spread trade version 1: long 2nd nearby, short 3rd nearby
  tIV["LST1N2", "InitialValue"] <- tIV["LSFN2", "InitialValue"] +  tIV["SSFN3", "InitialValue"]
  tIV["LST1N2", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LST1N2", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Long spread trade version 1: long 3rd nearby, short 4th nearby
  tIV["LST1N3", "InitialValue"] <- tIV["LSFN3", "InitialValue"] +  SSF4
  tIV["LST1N3", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LST1N3", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short spread trade version 1: short 1st nearby, long 2nd nearby
  tIV["SST1N1", "InitialValue"] <- -tIV["LST1N1", "InitialValue"]
  tIV["SST1N1", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SST1N1", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short spread trade version 1: short 2nd nearby, long 3rd nearby
  tIV["SST1N2", "InitialValue"] <- -tIV["LST1N2", "InitialValue"]
  tIV["SST1N2", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SST1N2", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short spread trade version 1: short 3rd nearby, long 4th nearby
  tIV["SST1N3", "InitialValue"] <- -tIV["LST1N3", "InitialValue"]
  tIV["SST1N3", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SST1N3", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Long spread trade version 2: long 1st nearby, short 3rd nearby
  FuturesInputData$TimeToMaturity <- tTTM + 0.25*4
  Value <- FVCA(FuturesInputData)
  SSF5 <- -FuturesInputData$Units * Value
  tIV["LST2N1", "InitialValue"] <- tIV["LSFN1", "InitialValue"] +  tIV["SSFN3", "InitialValue"]
  tIV["LST2N1", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LST2N1", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Long spread trade version 2: long 1st nearby, short 4th nearby
  tIV["LST2N2", "InitialValue"] <- tIV["LSFN1", "InitialValue"] +  SSF4
  tIV["LST2N2", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LST2N2", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Long spread trade version 2: long 1st nearby, short 5th nearby
  tIV["LST2N3", "InitialValue"] <- tIV["LSFN1", "InitialValue"] +  SSF5
  tIV["LST2N3", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LST2N3", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short spread trade 2: short 1st nearby, long 3rd nearby
  tIV["SST2N1", "InitialValue"] <- -tIV["LST2N1", "InitialValue"]
  tIV["SST2N1", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SST2N1", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short spread trade 2: short 1st nearby, long 4th nearby
  tIV["SST2N2", "InitialValue"] <- -tIV["LST2N2", "InitialValue"]
  tIV["SST2N2", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SST2N2", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
# Short spread trade 2: short 1st nearby, long 5th nearby
  tIV["SST2N3", "InitialValue"] <- -tIV["LST2N3", "InitialValue"]
  tIV["SST2N3", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SST2N3", "InitialValue"], FuturesInputData$Units*FuturesInputData$StockIndex))
  tIV <- round(tIV,4)
  FuturesInputData$TimeToMaturity <- tTTM
  return(tIV)
}
