# Stock Index Futures MCVaR Setup.R
# Stock 1
inputName <- 'ABC'
inputUnits <- 1                    # Number of futures contracts
inputStockIndex1 <- 100.0          # In dollars per futures contracts
inputLevelCC1 <- 4.0 #(3.0 - 2.0)  # In percent, annualized risk-free interest rate plus other carry costs (negative dividend yield),continuously compounded 
inputSlopeCC1 <- 0.0 #-(5.0 - 2.0) # In percent, annualized risk-free interest rate plus other carry costs (negative dividend yield),continuously compounded 
inputScalar1 <- 0.5                # Scalar used in the LSC model (Scalar 1)
inputTimeToMaturity1 = 0.25        # In years, time to NEARBY maturity (default: Quarterly, 0.25 or less)
inputType <- 1L                    # 1 for quarterly (default)
inputECC <- 10                     # Equivalent cash collateral percentage, in %
#
# Futures list
#
FuturesInputData <- list(inputName, inputUnits, inputStockIndex1, inputLevelCC1, 
  inputSlopeCC1, inputScalar1, inputTimeToMaturity1, inputType, inputECC)
names(FuturesInputData) <- c("Name", "Units", "StockIndex", "LevelCC", 
  "SlopeCC", "Scalar", "TimeToMaturity", "Type", "ECC")
dfFuturesInputData <- as.data.frame(FuturesInputData) # Used later to track inputs
# Stock 2 (eventually build out, or make assignment)
# Simulation input (only one stock for now)
inputHorizon <- 1/12 # Default (1 Week), must be less than TimeToMaturity
inputConfidenceLevel <- 99.0 # Percent
inputNumberOfSimulations <- 100000 # Default: 5,000 (100,000 because relatively fast)
inputStockMean <- 10.0     # In percent, expected percentage change in stock, continuously compounded, annualized
inputLCCMean <- 0.0    # In percent, expected percentage change in interest rate, cc
inputSCCMean <- 0.0    # In percent, expected percentage change in SD, cc
inputStockSD <- 20.0      # In percent, continuously compounded, annualized
inputLCCSD <- 10.0      # In percent, continuously compounded, annualized
inputSCCSD <- 20.0      # In percent, continuously compounded, annualized
inputCorLCCSCC <- -0.5  # Correlation between changes in rates and changes in vol
inputCorStockLCC <- -0.3 # Correlation between stock returns and changes in rates
inputCorStockSCC <- 0.5  # Correlation between stock returns and changes in SD
SimulationInputData <- list(inputName, inputHorizon, inputConfidenceLevel, 
  inputNumberOfSimulations, inputStockMean, inputLCCMean, 
  inputSCCMean, inputStockSD, inputLCCSD,
  inputSCCSD, inputCorLCCSCC, inputCorStockLCC, inputCorStockSCC)
names(SimulationInputData) <- c("Name", "Horizon", "ConfidenceLevel", 
  "NumberOfSimulations", "StockMean", "LCCMean", 
  "SCCMean", "StockSD", "LCCSD", 
  "SCCSD", "CorLCCSCC", "CorStockLCC", "CorStockSCC")
Increment <- (UpperBound - LowerBound)/(NumberOfStrategies - 1)
MCVaROutput = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NumberOfCorrelations))
colnames(MCVaROutput)  <- seq(1:NumberOfCorrelations)
# 19 Strategies compared:
#   1) (LS) Long stock
#   2) (LSF) Long stock index futures 
#      (N1 1st nearby, N2 2nd nearby, N3 3rd nearby)
#   3) (SSF) Short stock index futures (XL, X, XH)
#      (N1 1st nearby, N2 2nd nearby, N3 3rd nearby)
#   4) (LST1) Long spread trade 1 (long stock index futures, short next stock index futures) 
#      (N1 Long 1st nearby/Short 2nd nearby, N2 Long 2nd nearby/Short 3rd nearby, N3 Long 3rd nearby/Short 4th nearby)
#   5) (SST1) Short spread trade 1 (short stock index futures, long next stock index futures)
#      (N1 Short 1st nearby/Long 2nd nearby, N2 Short 2nd nearby/Long 3rd nearby, N3 Short 3rd nearby/Long 4th nearby)
#   6) (LST2) Long spread trade 2 (long 1 nearby stock index futures, short distant varying stock index futures) 
#      (N1 Long 1st nearby/Short 3rd nearby, N2 Long 1st nearby/Short 4th nearby, N3 Long 1st nearby/Short 5th nearby)
#   7) (SST2) Short spread trade 2 (short 1 nearby stock index futures, long distant varying stock index futures)
#      (N1 Short 1st nearby/Long 3rd nearby, N2 Short 1st nearby/Long 4th nearby, N3 Short 1st nearby/Long 5th nearby)
#
rownames(MCVaROutput)  <-  c("LS", # Long stock index
  "LSFN1", "LSFN2", "LSFN3", # Long stock index futures (nearby 1, 2, and 3)
  "SSFN1", "SSFN2", "SSFN3", # Short stock index futures (nearby 1, 2, and 3)
  "LST1N1", "LST1N2", "LST1N3", # Long spread trade (long nearby, short 2nd nearby ...)
  "SST1N1", "SST1N2", "SST1N3", # Short spread trade (short nearby, long 2nd nearby ...)
  "LST2N1", "LST2N2", "LST2N3", # Long spread trade 2 (long nearby, short 3rd nearby ...)
  "SST2N1", "SST2N2", "SST2N3" # Short spread trade 2 (short nearby, long 3rd nearby ...)
)         
MCRVaROutput <- MCVaROutput
# Data frame holding outputs of valuations
InitialValuations = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NumberOfSecurities + 1))
colnames(InitialValuations)  <- c("InitialValue", "Margin")
rownames(InitialValuations)  <- c("LS", # Long stock index
  "LSFN1", "LSFN2", "LSFN3", # Long stock index futures (nearby 1, 2, and 3)
  "SSFN1", "SSFN2", "SSFN3", # Short stock index futures (nearby 1, 2, and 3)
  "LST1N1", "LST1N2", "LST1N3", # Long spread trade (long nearby, short 2nd nearby ...)
  "SST1N1", "SST1N2", "SST1N3", # Short spread trade (short nearby, long 2nd nearby ...)
  "LST2N1", "LST2N2", "LST2N3", # Long spread trade 2 (long nearby, short 3rd nearby ...)
  "SST2N1", "SST2N2", "SST2N3" # Short spread trade 2 (short nearby, long 3rd nearby ...)
) 

