# Stock Index Futures MCVaR Test.R
# NOTE: This code sort of follows Module 12.1 GBM BOVM MCVaR Test.R
# The goal is to facilitate comparison.
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
# Libraries
#  MASS - Multivariate normal random number generator
Packages <- c("MASS","openxlsx", "Matrix", "matrixcalc", "beepr")
if(length(setdiff(Packages, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(Packages, rownames(installed.packages())))
} # Make sure libraries are installed on this computer
lapply(Packages, library, character.only=TRUE) # Load and attach libraries
rm(Packages)
op <- options(digits.secs = 6)
StartTime <- Sys.time()
LagtEndRunTime <- StartTime
# Build data frames of inputs
# Assumptions:
#  Quarterly futures contract
#  tauF - fraction of year until 1st nearby matures (tauF + 0.25*N (number of nearby))
#  NN - number of nearbys to analyze (default = 3)
# Note:
#      w = proportion of Equivalent Cash Collateral applied to HPR calculations
#      X = XL-1st nearby, X-2nd nearby, XH- 3rd nearby (evaluate only three)
# 19 Strategies compared:
#   1) Long stock
#   2) Long stock index futures 
#      (XL-L 1st nearby, X-L 2nd nearby, XH-L 3rd nearby)
#   3) Short stock index futures (XL, X, XH)
#      (XL-S 1st nearby, X-S 2nd nearby, XH-S 3rd nearby)
#   4) Long spread trade 1 (long stock index futures, short next stock index futures) 
#      (XL-L 1st nearby, S 2nd nearby, X-L 2nd nearby, S 3rd nearby, XH-L 3rd nearby, S 4th nearby)
#   5) Short spread trade 1 (short stock index futures, long next stock index futures)
#      (XL-S 1st nearby, L 2nd nearby, X-S 2nd nearby, L 3rd nearby, XH-S 3rd nearby, L 4th nearby)
#   6) Long spread trade 2 (long 2 stock index futures, short distant varying stock index futures) 
#      (XL-L 2 1st nearby, S 2nd nearby, X-L 2 1st nearby, S 3rd nearby, XH-L 2 1st nearby, S 4th nearby)
#   7) Short spread trade 2 (short 2 stock index futures, long distant varying stock index futures)
#      (XL-S 2 1st nearby, L 2nd nearby, X-S 2 1st nearby, L 3rd nearby, XH-S 2 1st nearby, L 4th nearby)
#   
#
# Study #1: Role of correlation between Stock and Level Carry Costs
#
LowerBound <- -0.75
UpperBound <- 0.75
NumberOfStrategies <- 19 # Long only for now
NumberOfCorrelations <- 7
NumberOfSecurities <- 1 # Analysis of only one underlying
source('Stock Index Futures MCVaR Core.R')
#
# Source all required functions
#
source("Stock Index Futures MCVaR Functions.R")
#
# Correlation Study
#
# Loop over all correlation cases
#   Variable <- "CorStockLCC"
#   Variable <- "CorStockSCC"
#   Variable <- "CorLCCSCC"

for(kk in 1:3){
  if(kk == 1){
    source('Stock Index Futures MCVaR Setup.R')
    Variable <- "CorStockLCC"
  } else if(kk == 2){
    source('Stock Index Futures MCVaR Setup.R')
    Variable <- "CorStockSCC"
  } else {
    source('Stock Index Futures MCVaR Setup.R')
    Variable <- "CorLCCSCC"
  }
# Sensitivity Variable
  Increment <- (UpperBound - LowerBound)/(NumberOfCorrelations - 1)
  SensitivityVariable = data.frame(matrix(NA, nrow = NumberOfCorrelations, ncol = 1))
  colnames(SensitivityVariable)  <- 'Parameter'
  rownames(SensitivityVariable)  <- seq(1:NumberOfCorrelations)
  for(i in 1:NumberOfCorrelations){
    SensitivityVariable$Parameter[i] <- LowerBound + (i - 1)*Increment
  }
  Tab <- "Sensitivity Variable"
  SensitivityVariable <- cbind(Parameters = rownames(SensitivityVariable), SensitivityVariable)
  rownames(SensitivityVariable) <- 1:nrow(SensitivityVariable)
  wb <- buildWorkbook(SensitivityVariable, sheetName = Tab, gridLines = TRUE)
  BASECASE <- FALSE
  if(Variable == "CorStockLCC"){
    tCorStockLCC <- SimulationInputData$CorStockLCC
  }
  if(Variable == "CorStockSCC"){
    tCorStockSCC <- SimulationInputData$CorStockSCC
  }
  if(Variable == "CorLCCSCC"){
    tCorLCCSCC <- SimulationInputData$CorLCCSCC
  }
  for(i in 1:NumberOfCorrelations){
    if(Variable == "CorStockLCC")SimulationInputData$CorStockLCC <- LowerBound + (i - 1)*Increment
    if(Variable == "CorStockSCC")SimulationInputData$CorStockSCC <- LowerBound + (i - 1)*Increment
    if(Variable == "CorLCCSCC")SimulationInputData$CorLCCSCC <- LowerBound + (i - 1)*Increment
    tMCOutput <- fSIFMCVaRCore(FuturesInputData, InitialValuations, Variable)
    MCVaROutput[,i] <- tMCOutput$VaR
    MCRVaROutput[,i] <- tMCOutput$RVaR
  }
  if(Variable == "CorStockLCC"){
    SimulationInputData$CorStockLCC <- tCorStockLCC
  }
  if(Variable == "CorStockSCC"){
    SimulationInputData$CorStockSCC <- tCorStockSCC
  }
  if(Variable == "CorLCCSCC"){
    SimulationInputData$CorLCCSCC <- tCorLCCSCC
  }
# VaR output based on sensitivities
  MCVaROutput <- cbind(Strategy = rownames(MCVaROutput), MCVaROutput)
  rownames(MCVaROutput) <- 1:nrow(MCVaROutput)
  Tab <- "MCVaR Output"
  addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
  writeData(wb, Tab, MCVaROutput)
# Return VaR output based on sensitivities
  MCRVaROutput <- cbind(Strategy = rownames(MCRVaROutput), MCRVaROutput)
  rownames(MCRVaROutput) <- 1:nrow(MCRVaROutput)
  Tab <- "MCRVaR Output"
  addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
  writeData(wb, Tab, MCRVaROutput)
  FilePath <- "Selected Output/"
  WorkBookName <- paste0(FilePath, 'SIF MC VaR and RVaR ', Variable, ' 1', '.xlsx')
  saveWorkbook(wb, WorkBookName, overwrite = TRUE)
  BASECASE <- TRUE
  Valuations <- fSIFMCVaRCore(FuturesInputData, InitialValuations, Variable)
  tEndRunTime <- Sys.time()
  inputNumberOfSimulations
  tRunTime <- tEndRunTime - LagtEndRunTime
  LagtEndRunTime <- tEndRunTime
  print(tRunTime)
}
#
# Margin Study
#
UpperBound <- 100 # 100% margin
LowerBound <- 10  # 10% margin
NumberOfMargins <- 10
NumberOfCorrelations <- NumberOfMargins # Avoid rewriting correlation code
source('Stock Index Futures MCVaR Setup.R')
Variable <- "Margin"
# Sensitivity Variable  
Increment <- (UpperBound - LowerBound)/(NumberOfMargins - 1)
SensitivityVariable = data.frame(matrix(NA, nrow = NumberOfMargins, ncol = 1))
colnames(SensitivityVariable)  <- 'Parameter'
rownames(SensitivityVariable)  <- seq(1:NumberOfMargins)
for(i in 1:NumberOfMargins){
  SensitivityVariable$Parameter[i] <- LowerBound + (i - 1)*Increment
}
Tab <- "Sensitivity Variable"
SensitivityVariable <- cbind(Parameters = rownames(SensitivityVariable), SensitivityVariable)
rownames(SensitivityVariable) <- 1:nrow(SensitivityVariable)
wb <- buildWorkbook(SensitivityVariable, sheetName = Tab, gridLines = TRUE)
BASECASE <- FALSE
tECC <- FuturesInputData$ECC
for(i in 1:NumberOfMargins){
  FuturesInputData$ECC <- LowerBound + (i - 1)*Increment
  tMCOutput <- fSIFMCVaRCore(FuturesInputData, InitialValuations, Variable)
  MCVaROutput[,i] <- tMCOutput$VaR
  MCRVaROutput[,i] <- tMCOutput$RVaR
}
FuturesInputData$ECC <- tECC
# VaR output based on sensitivities
MCVaROutput <- cbind(Strategy = rownames(MCVaROutput), MCVaROutput)
rownames(MCVaROutput) <- 1:nrow(MCVaROutput)
Tab <- "MCVaR Output"
addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
writeData(wb, Tab, MCVaROutput)
# Return VaR output based on sensitivities
MCRVaROutput <- cbind(Strategy = rownames(MCRVaROutput), MCRVaROutput)
rownames(MCRVaROutput) <- 1:nrow(MCRVaROutput)
Tab <- "MCRVaR Output"
addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
writeData(wb, Tab, MCRVaROutput)
FilePath <- "Selected Output/"
WorkBookName <- paste0(FilePath, 'SIF MC VaR and RVaR ', Variable, ' 1', '.xlsx')
saveWorkbook(wb, WorkBookName, overwrite = TRUE)
BASECASE <- TRUE
Valuations <- fSIFMCVaRCore(FuturesInputData, InitialValuations, Variable)
tEndRunTime <- Sys.time()
inputNumberOfSimulations
tRunTime <- tEndRunTime - LagtEndRunTime
LagtEndRunTime <- tEndRunTime
print(tRunTime)
# Entire program
EndTime <- Sys.time()
inputNumberOfSimulations
RunTime <- EndTime - StartTime
RunTime
beep(sound = 2, print('Finished'))
