# Stock Index MCVaR Functions.R
#
source("FCAVM With SRM Functions.R")
#
# Portfolio risk management functions
#
# Set initial values for 19 positions

fInitialValues = function(FuturesInputData, InitialValuations){
  tIV <- InitialValuations
  tTTM1 <- FuturesInputData$TimeToMaturity1
# Long underlying instrument 1
  tIV["LUI1", "InitialValue"] <- FuturesInputData$Units1 * FuturesInputData$UI1
  tIV["LUI1", "Margin"] <- (FuturesInputData$ECC/100)*abs(tIV["LUI1", "InitialValue"])
# Long underlying instrument 1, short 1st nearby futures 2
# Initialize temporary futures input data for valuation only  
  tUI <- FuturesInputData$UI2
  tLCC <- FuturesInputData$LevelCC2
  tSCC <- FuturesInputData$SlopeCC2
  tTTM <- FuturesInputData$TimeToMaturity2
  tScalar <- FuturesInputData$Scalar1
  FID <- list(tUI, tLCC, tSCC, tTTM, tScalar)
  names(FID) <- c("UI", "LevelCC", "SlopeCC", "TimeToMaturity", "Scalar")
# Long underlying instrument 1, short 1st nearby futures 2
  FID$TimeToMaturity <- tTTM
  Value <-  FVCA(FID)
  tIV["LUI1SF21", "InitialValue"] <- tIV["LUI1", "InitialValue"] - FuturesInputData$Units2 * Value
  tIV["LUI1SF21", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LUI1SF21", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long underlying instrument 1, short 2nd nearby futures 2
  FID$TimeToMaturity <- tTTM + 0.25
  Value <- FVCA(FID)
  tIV["LUI1SF22", "InitialValue"] <- tIV["LUI1", "InitialValue"] - FuturesInputData$Units2 * Value
  tIV["LUI1SF22", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LUI1SF22", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long underlying instrument 1, short 3rd nearby futures 3
  FID$TimeToMaturity <- tTTM + 0.5
  Value <- FVCA(FID)
  tIV["LUI1SF23", "InitialValue"] <- tIV["LUI1", "InitialValue"] - FuturesInputData$Units2 * Value
  tIV["LUI1SF23", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LUI1SF23", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Short underlying instrument 1, long 1st nearby futures 2
  tIV["SUI1LF21", "InitialValue"] <- -tIV["LUI1SF21", "InitialValue"]
  tIV["SUI1LF21", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SUI1LF21", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long underlying instrument 1, short 2nd nearby futures 2
  tIV["SUI1LF22", "InitialValue"] <- -tIV["LUI1SF22", "InitialValue"]
  tIV["SUI1LF22", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SUI1LF22", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long underlying instrument 1, short 3rd nearby futures 3
  tIV["SUI1LF23", "InitialValue"] <- -tIV["LUI1SF23", "InitialValue"]
  tIV["SUI1LF23", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SUI1LF23", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long 1st nearby futures 1, short 1st nearby futures 2
  FID$UI <- FuturesInputData$UI1
  FID$LevelCC <- FuturesInputData$LevelCC1
  FID$SlopeCC <- FuturesInputData$SlopeCC1
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity1
  FID$Scalar <- FuturesInputData$Scalar1
  Value1 <-  FVCA(FID)
  FID$UI <- FuturesInputData$UI2
  FID$LevelCC <- FuturesInputData$LevelCC2
  FID$SlopeCC <- FuturesInputData$SlopeCC2
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity2
  FID$Scalar <- FuturesInputData$Scalar1 # Number is in LSC model not futures contracts
  Value2 <-  FVCA(FID)
  tIV["LF11SF21", "InitialValue"] <- Value1 - Value2
  tIV["LF11SF21", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LF11SF21", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long 2nd nearby futures 1, short 2nd nearby futures 2
  FID$UI <- FuturesInputData$UI1
  FID$LevelCC <- FuturesInputData$LevelCC1
  FID$SlopeCC <- FuturesInputData$SlopeCC1
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity1 + 0.25
  FID$Scalar <- FuturesInputData$Scalar1
  Value1 <-  FVCA(FID)
  FID$UI <- FuturesInputData$UI2
  FID$LevelCC <- FuturesInputData$LevelCC2
  FID$SlopeCC <- FuturesInputData$SlopeCC2
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity2 + 0.25
  FID$Scalar <- FuturesInputData$Scalar1 # Number is in LSC model not futures contracts
  Value2 <-  FVCA(FID)
  tIV["LF12SF22", "InitialValue"] <- Value1 - Value2
  tIV["LF12SF22", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LF12SF22", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long 3rd nearby futures 1, short 3rd nearby futures 2
  FID$UI <- FuturesInputData$UI1
  FID$LevelCC <- FuturesInputData$LevelCC1
  FID$SlopeCC <- FuturesInputData$SlopeCC1
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity1 + 0.5
  FID$Scalar <- FuturesInputData$Scalar1
  Value1 <-  FVCA(FID)
  FID$UI <- FuturesInputData$UI2
  FID$LevelCC <- FuturesInputData$LevelCC2
  FID$SlopeCC <- FuturesInputData$SlopeCC2
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity2 + 0.5
  FID$Scalar <- FuturesInputData$Scalar1 # Number is in LSC model not futures contracts
  Value2 <-  FVCA(FID)
  tIV["LF13SF23", "InitialValue"] <- Value1 - Value2
  tIV["LF13SF23", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LF13SF23", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Short 1st nearby futures 1, long 1st nearby futures 2
  tIV["SF11LF21", "InitialValue"] <- -tIV["LF11SF21", "InitialValue"]
  tIV["SF11LF21", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SF11LF21", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Short 2nd nearby futures 1, long 2nd nearby futures 2
  tIV["SF12LF22", "InitialValue"] <- -tIV["LF12SF22", "InitialValue"]
  tIV["SF12LF22", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SF12LF22", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long 3rd nearby futures 1, short 3rd nearby futures 2
  tIV["SF13LF23", "InitialValue"] <- -tIV["LF13SF23", "InitialValue"]
  tIV["SF13LF23", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SF13LF23", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long 1st nearby futures 1, short 2nd nearby futures 2
  FID$UI <- FuturesInputData$UI1
  FID$LevelCC <- FuturesInputData$LevelCC1
  FID$SlopeCC <- FuturesInputData$SlopeCC1
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity1
  FID$Scalar <- FuturesInputData$Scalar1
  Value1 <-  FVCA(FID)
  FID$UI <- FuturesInputData$UI2
  FID$LevelCC <- FuturesInputData$LevelCC2
  FID$SlopeCC <- FuturesInputData$SlopeCC2
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity2 + 0.25
  FID$Scalar <- FuturesInputData$Scalar1 # Number is in LSC model not futures contracts
  Value2 <-  FVCA(FID)
  tIV["LF11SF22", "InitialValue"] <- Value1 - Value2
  tIV["LF11SF22", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LF11SF22", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long 1st nearby futures 1, short 3rd nearby futures 2
  FID$UI <- FuturesInputData$UI1
  FID$LevelCC <- FuturesInputData$LevelCC1
  FID$SlopeCC <- FuturesInputData$SlopeCC1
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity1
  FID$Scalar <- FuturesInputData$Scalar1
  Value1 <-  FVCA(FID)
  FID$UI <- FuturesInputData$UI2
  FID$LevelCC <- FuturesInputData$LevelCC2
  FID$SlopeCC <- FuturesInputData$SlopeCC2
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity2 + 0.5
  FID$Scalar <- FuturesInputData$Scalar1 # Number is in LSC model not futures contracts
  Value2 <-  FVCA(FID)
  tIV["LF11SF23", "InitialValue"] <- Value1 - Value2
  tIV["LF11SF23", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LF11SF23", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Long 1st nearby futures 1, short 4th nearby futures 2
  FID$UI <- FuturesInputData$UI1
  FID$LevelCC <- FuturesInputData$LevelCC1
  FID$SlopeCC <- FuturesInputData$SlopeCC1
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity1
  FID$Scalar <- FuturesInputData$Scalar1
  Value1 <-  FVCA(FID)
  FID$UI <- FuturesInputData$UI2
  FID$LevelCC <- FuturesInputData$LevelCC2
  FID$SlopeCC <- FuturesInputData$SlopeCC2
  FID$TimeToMaturity <- FuturesInputData$TimeToMaturity2 + 0.75
  FID$Scalar <- FuturesInputData$Scalar1 # Number is in LSC model not futures contracts
  Value2 <-  FVCA(FID)
  tIV["LF11SF24", "InitialValue"] <- Value1 - Value2
  tIV["LF11SF24", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["LF11SF24", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Short 1st nearby futures 1, long 2nd nearby futures 2
  tIV["SF11LF22", "InitialValue"] <- -tIV["LF11SF22", "InitialValue"]
  tIV["SF11LF22", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SF11LF22", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Short 1st nearby futures 1, long 3rd nearby futures 2
  tIV["SF11LF23", "InitialValue"] <- -tIV["LF11SF23", "InitialValue"]
  tIV["SF11LF23", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SF11LF23", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
# Short 1st nearby futures 1, long 4th nearby futures 2
  tIV["SF11LF24", "InitialValue"] <- -tIV["LF11SF24", "InitialValue"]
  tIV["SF11LF24", "Margin"] <- (FuturesInputData$ECC/100) *
    abs(max(tIV["SF11LF24", "InitialValue"], FuturesInputData$Units1*FuturesInputData$UI1))
  return(tIV)
}


