# Selected Other Futures MCVaR Setup.R
# Two underlying instruments
inputName1 <- 'ABC'           # Underlying name 1
inputName2 <- 'DEF'           # Underlying name 2
inputUnits1 <- 1              # Number of units within each futures contract 1
inputUnits2 <- 1              # Number of units within each futures contract 2
inputUI1 <- 100.0             # Underlying instrumennt 1, in dollars per futures contracts
inputUI2 <- 100.0             # Underlying instrument 2, in dollars per futures contracts
inputLevelCC1 <- 4.0          # In percent, annualized risk-free interest rate plus other carry costs (negative dividend yield),continuously compounded 
inputLevelCC2 <- -2.0          # In percent, annualized risk-free interest rate plus other carry costs (negative dividend yield),continuously compounded 
inputSlopeCC1 <- 0.0          # In percent, annualized risk-free interest rate plus other carry costs (negative dividend yield),continuously compounded 
inputSlopeCC2 <- 0.0          # In percent, annualized risk-free interest rate plus other carry costs (negative dividend yield),continuously compounded 
inputScalar1 <- 0.5           # Scalar used in both LSC model (Two factor LSC, thus only one scalar 1)
inputTimeToMaturity1 = 0.25   # In years, time to NEARBY maturity (default: Quarterly, 0.25 or less)
inputTimeToMaturity2 = 0.25   # In years, time to NEARBY maturity (default: Quarterly, 0.25 or less)
inputType <- 1L               # 1 for quarterly (default)
inputECC <- 10                # Equivalent cash collateral percentage, in %
#
# Futures list
#
FuturesInputData <- list(inputName1, inputName2, inputUnits1, inputUnits2, 
  inputUI1, inputUI2, inputLevelCC1, inputLevelCC2, inputSlopeCC1, inputSlopeCC2, 
  inputScalar1, inputTimeToMaturity1, inputTimeToMaturity2, inputType, inputECC)
names(FuturesInputData) <- c("Name1", "Name2", "Units1", "Units2", "UI1", "UI2", 
  "LevelCC1", "LevelCC2", "SlopeCC1", "SlopeCC2", "Scalar1", 
  "TimeToMaturity1", "TimeToMaturity2", "Type", "ECC")
dfFuturesInputData <- as.data.frame(FuturesInputData) # Used later to track inputs
# Simulation input (only one stock for now)
inputHorizon <- 1/12 # Default (1 Week), must be less than TimeToMaturity
inputConfidenceLevel <- 99.0 # Percent
inputNumberOfSimulations <- 100000 # Default: 5,000 (100,000 because relatively fast)
#
# Two underlyings are assumed to capture inter-market spread risk
#
inputUI1Mean <- 10.0     # In percent, expected percentage change in underlying 1, continuously compounded, annualized (cca)
inputUI2Mean <- 10.0     # In percent, expected percentage change in underlying 2, cca
inputLCC1Mean <- 0.0    # In percent, expected percentage change in level carry cost for underlying 1, cca
inputLCC2Mean <- 0.0    # In percent, expected percentage change in level carry cost for underlying 2, cca
inputSCC1Mean <- 0.0    # In percent, expected percentage change in slope carry cost for underlying 1, cca
inputSCC2Mean <- 0.0    # In percent, expected percentage change in slope carry cost for underlying 2, cca
inputUI1SD <- 20.0      # In percent, standard deviation of percentage change in underlying 1, cca
inputUI2SD <- 20.0      # In percent, standard deviation of percentage change in underlying 2, cca
inputLCC1SD <- 10.0      # In percent, standard deviation of changes in level for underlying 1, cca
inputLCC2SD <- 10.0      # In percent, standard deviation of changes in level for underlying 2, cca
inputSCC1SD <- 20.0      # In percent, standard deviation of changes in slope for underlying 1, cca
inputSCC2SD <- 20.0      # In percent, standard deviation of changes in slope for underlying 2, cca
inputCorUI1UI2 <- 0.8 # Correlation between stock returns
inputCorUI1LCC1 <- -0.3 # Correlation between stock return 1 and level 1
inputCorUI1LCC2 <- -0.3 # Correlation between stock return 1 and level 2
inputCorUI1SCC1 <- 0.5  # Correlation between stock return 1 and slope 1
inputCorUI1SCC2 <- 0.5  # Correlation between stock return 1 and slope 2
inputCorUI2LCC1 <- -0.3 # Correlation between stock return 2 and level 1
inputCorUI2LCC2 <- -0.3 # Correlation between stock return 2 and level 2
inputCorUI2SCC1 <- 0.5  # Correlation between stock return 2 and slope 1
inputCorUI2SCC2 <- 0.5  # Correlation between stock return 2 and slope 2
inputCorLCC1LCC2 <- 0.8 # Correlation between level 1 and level 2
inputCorLCC1SCC1 <- 0.7  # Correlation between level 1 and slope 1
inputCorLCC1SCC2 <- 0.6  # Correlation between level 1 and slope 2
inputCorLCC2SCC1 <- 0.7  # Correlation between level 2 and slope 1
inputCorLCC2SCC2 <- 0.6  # Correlation between level 2 and slope 2
inputCorSCC1SCC2 <- 0.6  # Correlation between slope 1 and slope 2
SimulationInputData <- list(inputName1, inputName2, inputHorizon, inputConfidenceLevel, inputNumberOfSimulations, 
  inputUI1Mean, inputUI2Mean, inputLCC1Mean, inputLCC2Mean, inputSCC1Mean, inputSCC2Mean, 
  inputUI1SD, inputUI2SD, inputLCC1SD, inputLCC2SD, inputSCC1SD, inputSCC2SD, 
  inputCorUI1UI2, inputCorUI1LCC1, inputCorUI1LCC2, inputCorUI1SCC1, inputCorUI1SCC2, 
  inputCorUI2LCC1, inputCorUI2LCC2, inputCorUI2SCC1, inputCorUI2SCC2, 
  inputCorLCC1LCC2, inputCorLCC1SCC1, inputCorLCC1SCC2, 
  inputCorLCC2SCC1, inputCorLCC2SCC2, 
  inputCorSCC1SCC2)
names(SimulationInputData) <- c("Name1", "Name2", "Horizon", "ConfidenceLevel", "NumberOfSimulations", 
  "UI1Mean", "UI2Mean", "LCC1Mean", "LCC2Mean", "SCC1Mean", "SCC2Mean", 
  "UI1SD", "UI2SD", "LCC1SD", "LCC2SD", "SCC1SD", "SCC2SD", 
  "CorUI1UI2", "CorUI1LCC1", "CorUI1LCC2", "CorUI1SCC1", "CorUI1SCC2", 
  "CorUI2LCC1", "CorUI2LCC2", "CorUI2SCC1", "CorUI2SCC2", 
  "CorLCC1LCC2", "CorLCC1SCC1", "CorLCC1SCC2", 
  "CorLCC2SCC1", "CorLCC2SCC2", 
  "CorSCC1SCC2")
Increment <- (UpperBound - LowerBound)/(NumberOfStrategies - 1)
MCVaROutput = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NumberOfCorrelations))
colnames(MCVaROutput)  <- seq(1:NumberOfCorrelations)
# 19 Strategies compared:
#
# Assume UI1 held, futures-based on UI2
#
#   1) (LUI1) Long underlying instrument 1
#   2) (LUI1SF21) Long underlying instrument 1, short futures 2 nearby 1 
#   3) (LUI1SF22) Long underlying instrument 1, short futures 2 nearby 2 
#   4) (LUI1SF23) Long underlying instrument 1, short futures 2 nearby 3
#   5) (SUI1LF21) Short underlying instrument 1, long futures 2 nearby 1 
#   6) (SUI1LF22) Short underlying instrument 1, long futures 2 nearby 2 
#   7) (SUI1LF23) Short underlying instrument 1, long futures 2 nearby 3
#   8) (LF11SF21) Long futures 1 nearby 1, short futures 2 nearby 1
#   9) (LF12SF22) Long futures 1 nearby 2, short futures 2 nearby 2
#   10) (LF13SF23) Long futures 1 nearby 3, short futures 2 nearby 3
#   11) (SF11SF21) Short futures 1 nearby 1, long futures 2 nearby 1
#   12) (SF12SF22) Short futures 1 nearby 2, long futures 2 nearby 2
#   13) (SF13SF23) Short futures 1 nearby 3, long futures 2 nearby 3
#   14) (LF11SF22) Long futures 1 nearby 1, short futures 2 nearby 2
#   15) (LF11SF23) Long futures 1 nearby 1, short futures 2 nearby 3
#   16) (LF11SF24) Long futures 1 nearby 1, short futures 2 nearby 4
#   17) (SF11LF22) Short futures 1 nearby 1, long futures 2 nearby 2
#   18) (SF11LF23) Short futures 1 nearby 1, long futures 2 nearby 3
#   19) (SF11LF24) Short futures 1 nearby 1, long futures 2 nearby 4
rownames(MCVaROutput)  <-  c("LUI1", 
  "LUI1SF21", "LUI1SF22", "LUI1SF23", # Long UI1, short futures 2 (nearby 1, 2, and 3)
  "SUI1LF21", "SUI1LF22", "SUI1LF23", # Short UI1, long futures 2 (nearby 1, 2, and 3)
  "LF11SF21", "LF12SF22", "LF13SF23", # Long futures 1, short futures 2 (same maturity)
  "SF11LF21", "SF12LF22", "SF13LF23", # Short futures 1, long futures 2 (same maturity)
  "LF11SF22", "LF11SF23", "LF11SF24", # Long futures 1, short futures 2 (longer maturities)
  "SF11LF22", "SF11LF23", "SF11LF24" # Short futures 1, long futures 2 (longer maturities)
)         
MCRVaROutput <- MCVaROutput
# Data frame holding outputs
InitialValuations = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = 2))
colnames(InitialValuations)  <- c("InitialValue", "Margin")
rownames(InitialValuations)  <- c("LUI1", 
  "LUI1SF21", "LUI1SF22", "LUI1SF23", # Long UI1, short futures 2 (nearby 1, 2, and 3)
  "SUI1LF21", "SUI1LF22", "SUI1LF23", # Short UI1, long futures 2 (nearby 1, 2, and 3)
  "LF11SF21", "LF12SF22", "LF13SF23", # Long futures 1, short futures 2 (same maturity)
  "SF11LF21", "SF12LF22", "SF13LF23", # Short futures 1, long futures 2 (same maturity)
  "LF11SF22", "LF11SF23", "LF11SF24", # Long futures 1, short futures 2 (longer maturities)
  "SF11LF22", "SF11LF23", "SF11LF24" # Short futures 1, long futures 2 (longer maturities)
)         

