# Selected Other Futures MCVaR Test.R
# NOTE: This code sort of follows Module 12.1 GBM BOVM MCVaR Test.R
# The goal is to facilitate comparison.
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
# Libraries
#  MASS - Multivariate normal random number generator
Packages <- c("MASS","openxlsx", "Matrix", "matrixcalc", "beepr")
if(length(setdiff(Packages, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(Packages, rownames(installed.packages())))
} # Make sure libraries are installed on this computer
lapply(Packages, library, character.only=TRUE) # Load and attach libraries
rm(Packages)
op <- options(digits.secs = 6)
StartTime <- Sys.time()
LagtEndRunTime <- StartTime
# Build data frames of inputs
# Assumptions:
#  Quarterly futures contract
#  tauF - fraction of year until 1st nearby matures (tauF + 0.25*N (number of nearby))
#  NN - number of nearbys to analyze (default = 3)
# Note:
#      w = proportion of Equivalent Cash Collateral applied to HPR calculations
#      X = XL-1st nearby, X-2nd nearby, XH- 3rd nearby (evaluate only three)
# 19 Strategies compared:
#   1) Long stock
#   2) Long Other futures 
#      (XL-L 1st nearby, X-L 2nd nearby, XH-L 3rd nearby)
#   3) Short Other futures (XL, X, XH)
#      (XL-S 1st nearby, X-S 2nd nearby, XH-S 3rd nearby)
#   4) Long spread trade 1 (long Other futures, short next Other futures) 
#      (XL-L 1st nearby, S 2nd nearby, X-L 2nd nearby, S 3rd nearby, XH-L 3rd nearby, S 4th nearby)
#   5) Short spread trade 1 (short Other futures, long next Other futures)
#      (XL-S 1st nearby, L 2nd nearby, X-S 2nd nearby, L 3rd nearby, XH-S 3rd nearby, L 4th nearby)
#   6) Long spread trade 2 (long 2 Other futures, short distant varying Other futures) 
#      (XL-L 2 1st nearby, S 2nd nearby, X-L 2 1st nearby, S 3rd nearby, XH-L 2 1st nearby, S 4th nearby)
#   7) Short spread trade 2 (short 2 Other futures, long distant varying Other futures)
#      (XL-S 2 1st nearby, L 2nd nearby, X-S 2 1st nearby, L 3rd nearby, XH-S 2 1st nearby, L 4th nearby)
#   
#
# Study #1: Role of correlation between Stock and Level Carry Costs
#
LowerBound <- -0.75
UpperBound <- 0.75
NumberOfStrategies <- 19 # Long only for now
NumberOfCorrelations <- 7
NumberOfSecurities <- 2 # Analysis of only two underlyings
source('Selected Other Futures MCVaR Setup.R')
source('Selected Other Futures MCVaR Core.R')
#
# Source all required functions
#
source("Selected Other Futures MCVaR Functions.R")
#
# Correlation Study
#
# Loop over all correlation cases
#   Variable <- "CorUIUF"
#   Variable <- "CorUILCC"
#   Variable <- "CorUISCC"
#   Variable <- "CorUFLCC"
#   Variable <- "CorUFSCC"
#   Variable <- "CorLCCSCC"
for(kk in 1:15){
  if(kk == 1){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI1UI2"
  } else if(kk == 2){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI1LCC1"
  } else if(kk == 3){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI1LCC2"
  } else if(kk == 4){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI1SCC1"
  } else if(kk == 5){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI1SCC2"
  } else if(kk == 6){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI2LCC1"
  } else if(kk == 7){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI2LCC2"
  } else if(kk == 8){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI2SCC1"
  } else if(kk == 9){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorUI2SCC2"
  } else if(kk == 10){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorLCC1LCC2"
  } else if(kk == 11){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorLCC1SCC1"
  } else if(kk == 12){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorLCC1SCC2"
  } else if(kk == 13){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorLCC2SCC1"
  } else if(kk == 14){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorLCC2SCC2"
  } else if(kk == 15){
    source('Selected Other Futures MCVaR Setup.R')
    Variable <- "CorSCC1SCC2"
  }
# Sensitivity Variable: NumberOfCorrelations in table not random variable related
  Increment <- (UpperBound - LowerBound)/(NumberOfCorrelations - 1)
  SensitivityVariable = data.frame(matrix(NA, nrow = NumberOfCorrelations, ncol = 1))
  colnames(SensitivityVariable)  <- 'Parameter'
  rownames(SensitivityVariable)  <- seq(1:NumberOfCorrelations)
  for(i in 1:NumberOfCorrelations){
    SensitivityVariable$Parameter[i] <- LowerBound + (i - 1)*Increment
  }
  Tab <- "Sensitivity Variable"
  SensitivityVariable <- cbind(Parameters = rownames(SensitivityVariable), SensitivityVariable)
  rownames(SensitivityVariable) <- 1:nrow(SensitivityVariable)
  wb <- buildWorkbook(SensitivityVariable, sheetName = Tab, gridLines = TRUE)
  BASECASE <- FALSE
  if(Variable == "CorUI1UI2"){
    tCorUI1UI2 <- SimulationInputData$CorUI1UI2
  } 
  if(Variable == "CorUI1LCC1"){
    tCorUI1LCC1 <- SimulationInputData$CorUI1LCC1
  } 
  if(Variable == "CorUI1LCC2"){
    tCorUI1LCC2 <- SimulationInputData$CorUI1LCC2
  } 
  if(Variable == "CorUI1SCC1"){
    tCorUI1SCC1 <- SimulationInputData$CorUI1SCC1
  } 
  if(Variable == "CorUI1SCC2"){
    tCorUI1SCC2 <- SimulationInputData$CorUI1SCC2
  } 
  if(Variable == "CorUI2LCC1"){
    tCorUI2LCC1 <- SimulationInputData$CorUI2LCC1
  } 
  if(Variable == "CorUI2LCC2"){
    tCorUI2LCC2 <- SimulationInputData$CorUI2LCC2
  } 
  if(Variable == "CorUI2SCC1"){
    tCorUI2SCC1 <- SimulationInputData$CorUI2SCC1
  } 
  if(Variable == "CorUI2SCC2"){
    tCorUI2SCC2 <- SimulationInputData$CorUI2SCC2
  } 
  if(Variable == "CorLCC1LCC2"){
    tCorLCC1LCC2 <- SimulationInputData$CorLCC1LCC2
  } 
  if(Variable == "CorLCC1SCC1"){
    tCorLCC1SCC1 <- SimulationInputData$CorLCC1SCC1
  } 
  if(Variable == "CorLCC1SCC2"){
    tCorLCC1SCC2 <- SimulationInputData$CorLCC1SCC2
  } 
  if(Variable == "CorLCC2SCC1"){
    tCorLCC2SCC1 <- SimulationInputData$CorLCC2SCC1
  } 
  if(Variable == "CorLCC2SCC2"){
    tCorLCC2SCC2 <- SimulationInputData$CorLCC2SCC2
  } 
  if(Variable == "CorSCC1SCC2"){
    tCorSCC1SCC2 <- SimulationInputData$CorSCC1SCC2
  } 
  for(i in 1:NumberOfCorrelations){
    if(Variable == "CorUI1UI2")SimulationInputData$CorUI1UI2 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorUI1LCC1")SimulationInputData$CorUI1LCC1 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorUI1LCC2")SimulationInputData$CorUI1LCC2 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorUI1SCC1")SimulationInputData$CorUI1SCC1 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorUI1SCC2")SimulationInputData$CorUI1SCC2 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorUI2LCC1")SimulationInputData$CorUI2LCC1 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorUI2LCC2")SimulationInputData$CorUI2LCC2 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorUI2SCC1")SimulationInputData$CorUI2SCC1 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorUI2SCC2")SimulationInputData$CorUI2SCC2 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorLCC1LCC2")SimulationInputData$CorLCC1LCC2 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorLCC1SCC1")SimulationInputData$CorLCC1SCC1 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorLCC1SCC2")SimulationInputData$CorLCC1SCC2 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorLCC2SCC1")SimulationInputData$CorLCC2SCC1 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorLCC2SCC2")SimulationInputData$CorLCC2SCC2 <- LowerBound + (i - 1)*Increment
    if(Variable == "CorSCC1SCC2")SimulationInputData$CorSCC1SCC2 <- LowerBound + (i - 1)*Increment
    tMCOutput <- fSIFMCVaRCore(FuturesInputData, InitialValuations, Variable)
    MCVaROutput[,i] <- tMCOutput$VaR
    MCRVaROutput[,i] <- tMCOutput$RVaR
  }
# Restore to original  
  if(Variable == "CorUI1UI2")SimulationInputData$CorUI1UI2 <- tCorUI1UI2
  if(Variable == "CorUI1LCC1")SimulationInputData$CorUI1LCC1 <- tCorUI1LCC1
  if(Variable == "CorUI1LCC2")SimulationInputData$CorUI1LCC2 <- tCorUI1LCC2
  if(Variable == "CorUI1SCC1")SimulationInputData$CorUI1SCC1 <- tCorUI1SCC1
  if(Variable == "CorUI1SCC2")SimulationInputData$CorUI1SCC2 <- tCorUI1SCC2
  if(Variable == "CorUI2LCC1")SimulationInputData$CorUI2LCC1 <- tCorUI2LCC1
  if(Variable == "CorUI2LCC2")SimulationInputData$CorUI2LCC2 <- tCorUI2LCC2
  if(Variable == "CorUI2SCC1")SimulationInputData$CorUI2SCC1 <- tCorUI2SCC1
  if(Variable == "CorUI2SCC2")SimulationInputData$CorUI2SCC2 <- tCorUI2SCC2
  if(Variable == "CorLCC1LCC2")SimulationInputData$CorLCC1LCC2 <- tCorLCC1LCC2
  if(Variable == "CorLCC1SCC1")SimulationInputData$CorLCC1SCC1 <- tCorLCC1SCC1
  if(Variable == "CorLCC1SCC2")SimulationInputData$CorLCC1SCC2 <- tCorLCC1SCC2
  if(Variable == "CorLCC2SCC1")SimulationInputData$CorLCC2SCC1 <- tCorLCC2SCC1
  if(Variable == "CorLCC2SCC2")SimulationInputData$CorLCC2SCC2 <- tCorLCC2SCC2
  if(Variable == "CorSCC1SCC2")SimulationInputData$CorSCC1SCC2 <- tCorSCC1SCC2
# VaR output based on sensitivities
  MCVaROutput <- cbind(Strategy = rownames(MCVaROutput), MCVaROutput)
  rownames(MCVaROutput) <- 1:nrow(MCVaROutput)
  Tab <- "MCVaR Output"
  addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
  writeData(wb, Tab, MCVaROutput)
# Return VaR output based on sensitivities
  MCRVaROutput <- cbind(Strategy = rownames(MCRVaROutput), MCRVaROutput)
  rownames(MCRVaROutput) <- 1:nrow(MCRVaROutput)
  Tab <- "MCRVaR Output"
  addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
  writeData(wb, Tab, MCRVaROutput)
  FilePath <- "Selected Output/"
  WorkBookName <- paste0(FilePath, 'SIF MC VaR and RVaR ', Variable, ' 1', '.xlsx')
  saveWorkbook(wb, WorkBookName, overwrite = TRUE)
  BASECASE <- TRUE
  Valuations <- fSIFMCVaRCore(FuturesInputData, InitialValuations, Variable)
  tEndRunTime <- Sys.time()
  inputNumberOfSimulations
  tRunTime <- tEndRunTime - LagtEndRunTime
  LagtEndRunTime <- tEndRunTime
  print(tRunTime)
}
#
# Margin Study
#
UpperBound <- 100 # 100% margin
LowerBound <- 10  # 10% margin
NumberOfMargins <- 10
NumberOfCorrelations <- NumberOfMargins # Avoid rewriting correlation code
source('Selected Other Futures MCVaR Setup.R')
Variable <- "Margin"
# Sensitivity Variable
Increment <- (UpperBound - LowerBound)/(NumberOfMargins - 1)
SensitivityVariable = data.frame(matrix(NA, nrow = NumberOfMargins, ncol = 1))
colnames(SensitivityVariable)  <- 'Parameter'
rownames(SensitivityVariable)  <- seq(1:NumberOfMargins)
for(i in 1:NumberOfMargins){
  SensitivityVariable$Parameter[i] <- LowerBound + (i - 1)*Increment
}
Tab <- "Sensitivity Variable"
SensitivityVariable <- cbind(Parameters = rownames(SensitivityVariable), SensitivityVariable)
rownames(SensitivityVariable) <- 1:nrow(SensitivityVariable)
wb <- buildWorkbook(SensitivityVariable, sheetName = Tab, gridLines = TRUE)
BASECASE <- FALSE
tECC <- FuturesInputData$ECC
for(i in 1:NumberOfMargins){
  FuturesInputData$ECC <- LowerBound + (i - 1)*Increment
  tMCOutput <- fSIFMCVaRCore(FuturesInputData, InitialValuations, Variable)
  MCVaROutput[,i] <- tMCOutput$VaR
  MCRVaROutput[,i] <- tMCOutput$RVaR
}
FuturesInputData$ECC <- tECC
# VaR output based on sensitivities
MCVaROutput <- cbind(Strategy = rownames(MCVaROutput), MCVaROutput)
rownames(MCVaROutput) <- 1:nrow(MCVaROutput)
Tab <- "MCVaR Output"
addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
writeData(wb, Tab, MCVaROutput)
# Return VaR output based on sensitivities
MCRVaROutput <- cbind(Strategy = rownames(MCRVaROutput), MCRVaROutput)
rownames(MCRVaROutput) <- 1:nrow(MCRVaROutput)
Tab <- "MCRVaR Output"
addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
writeData(wb, Tab, MCRVaROutput)
FilePath <- "Selected Output/"
WorkBookName <- paste0(FilePath, 'SIF MC VaR and RVaR ', Variable, ' 1', '.xlsx')
saveWorkbook(wb, WorkBookName, overwrite = TRUE)
BASECASE <- TRUE
Valuations <- fSIFMCVaRCore(FuturesInputData, InitialValuations, Variable)
tEndRunTime <- Sys.time()
inputNumberOfSimulations
tRunTime <- tEndRunTime - LagtEndRunTime
LagtEndRunTime <- tEndRunTime
print(tRunTime)
# Entire program
EndTime <- Sys.time()
inputNumberOfSimulations
RunTime <- EndTime - StartTime
RunTime
beep(sound = 2, print('Finished'))
