# DRM Discount Curve Estimation With Simulation Test.R
# Model to illustrate MVN simulation of discount rates 
# rmarkdown::render("14.3 Discount Curve Estimation With Simulation Test.R", "word_document")
rm(list = ls())  # Take out the Environment "trash"
cat("\014") # Clear Console, making error checking easier.
while (!is.null(dev.list()))  dev.off() # Clear old plots
par(family = 'Times New Roman') # Globally set fonts for graphs
# Install requisite libraries: 
#   openxlsx - opening IRSwapBook.xlsx
PackagesToLibrary <- c("date", "optimx", "MASS", "openxlsx", "tis", "Matrix", 
  "matrixcalc", "beepr")
if (length(setdiff(PackagesToLibrary, rownames(installed.packages()))) > 0) {
  install.packages(setdiff(PackagesToLibrary, rownames(installed.packages())))  
} # Make sure libraries are installed on this computer
lapply(PackagesToLibrary, library, character.only = TRUE) # Load and attach libraries
rm(PackagesToLibrary)
op <- options(digits.secs = 6) # Monitor run times to identify reasonable 
StartTime <- Sys.time()
LagtEndRunTime <- StartTime
#
# 19 Strategies compared:
#   1) Base case (as inputted in spreadsheet)
#   2) Years: 1, 2, 3, 4, 5, 10, 20, 30 (8 strategies) 
#   3) Horizon: 1 day, 7 days, 61 days, 92 days (4 strategies)
#   4) Fixed Rate: -2, -1, +1, +2 percent (4 strategies)
#   5) Cash collateral +10, +20 percent from base case (2 strategies)
# 
# Numerous swap functions
source('SWAP Functions.R')
#
# Global inputs
#
Horizon <- 30 # Expressed in days
HorizonInYears <- Horizon/365
NumberOfSimulations <- 10000 # 50
FixRange <- TRUE
FRMax <- 10.0
FRMin <- 0.0
NF <- 3
ConfidenceLevel <- 95.0 # Percent
#
# Work on simulation based valuation
#
# Identify spreadsheet containing inputs
InputFileName <- 'DRM IRS Input File.xlsx'
source('DRM IRS Input Management.R')
source('SWAP Functions.R')
#
# New evaluation date
#
JulianEvaluationDate <- mdy.date(SVInputs$EM, SVInputs$ED, SVInputs$EY)
JulianHorizonDate <- as.integer(JulianEvaluationDate) + Horizon
HorizonInYears <- (as.numeric(JulianHorizonDate) - as.numeric(JulianEvaluationDate))/365.0
HDate <- date.mdy(JulianHorizonDate)
SimSWAP <- SVInputs
SimSWAP$HM <- HDate$month
SimSWAP$HD <- HDate$day
SimSWAP$HY <- HDate$year
SimSWAP$HorizonInYears <- HorizonInYears
SimSWAP$ConfidenceLevel <- ConfidenceLevel
SimSWAP$NumberOfSimulations <- NumberOfSimulations
SimSWAP$Output = 'Value1' # Two curves valuation method
# Simulation parameters
InputMVNName <- 'MVRN Parameters IRS Forward and Basis Curves.xlsx'
MVNSimulationParameters <- read.xlsx(InputMVNName, colNames = TRUE, rowNames = TRUE)
#
# Correlation study setup
#
LowerBound <- -0.75
UpperBound <- 0.75
NumberOfStrategies <- 19 
NumberOfCorrelations <- 3
#
# 19 Strategies compared:
#
# Assume UI1 held, futures-based on UI2
#
#   1) (SIC) Underlying swap (initial case)
#   2) (SMD1Y) Underlying swap with maturity date = 1 year
#   3) (SMD2Y) Underlying swap with maturity date = 2 years
#   4) (SMD3Y) Underlying swap with maturity date = 3 years
#   5) (SMD4Y) Underlying swap with maturity date = 4 years
#   6) (SMD5Y) Underlying swap with maturity date = 5 years
#   7) (SMD10Y) Underlying swap with maturity date = 10 years
#   8) (SMD20Y) Underlying swap with maturity date = 20 years
#   9) (SMD30Y) Underlying swap with maturity date = 30 years
#   10)	Underlying swap with fixed 30/360, Semi and floating ACT/360, Quart: (SDCPV)
#   11)	Underlying swap with both legs 30/360 Semi : (SDC30S)
#   12)	Underlying swap with both legs 30/360 Quart: (SDC30Q)
#   13)	Underlying swap with both legs ACT/360 (SDCACTQ)
#   14) (SFRM2) Underlying swap with 2 percent decrease in fixed rate
#   15) (SFRM1) Underlying swap with 1 percent decrease in fixed rate
#   16) (SFRP1) Underlying swap with 1 percent increase in fixed rate
#   17) (SFRP2) Underlying swap with 2 percent increase in fixed rate
#   18) (SCC10) Underlying swap with +10 percent point increase in cash collateral
#   19) (SCC20) Underlying swap with +20 percent point increase in cash collateral
#
MCVaROutput = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = NumberOfCorrelations))
colnames(MCVaROutput)  <- seq(1:NumberOfCorrelations)
rownames(MCVaROutput)  <-  c("SIC", 
  "SMD1Y", "SMD2Y", "SMD3Y", "SMD4Y", "SMD5Y", "SMD10Y", "SMD20Y", "SMD30Y", 
  "SDCPV", "SDC30S", "SDC30Q", "SDCACTQ",
  "SFRM2", "SFRM1", "SFRP1", "SFRP2", 
  "SCC10", "SCC20"
) 
MCRVaROutput <- MCVaROutput
tMCRVaROutput <- MCRVaROutput
tMCVaROutput <- MCVaROutput
# Data frame holding outputs
InitialValuations = data.frame(matrix(NA, nrow = NumberOfStrategies, ncol = 2))
colnames(InitialValuations)  <- c("InitialValue", "Collateral")
rownames(InitialValuations)  <- c("SIC", 
  "SMD1Y", "SMD2Y", "SMD3Y", "SMD4Y", "SMD5Y", "SMD10Y", "SMD20Y", "SMD30Y", 
  "SHD1D", "SHD7D", "SHD61D", "SHD92D",
  "SFRM2", "SFRM1", "SFRP1", "SFRP2", 
  "SCC10", "SCC20"
)   
tInitialValuations <- InitialValuations
# Temporary parameters that vary with simulator
tFRParm1 <- SimSWAP$FRParm1
tFRParm2 <- SimSWAP$FRParm2
tFRParm3 <- SimSWAP$FRParm3
tFRParm4 <- SimSWAP$FRParm4
tFRParm5 <- SimSWAP$FRParm5
tFRParm6 <- SimSWAP$FRParm6
tGRParm1 <- SimSWAP$GRParm1
tGRParm2 <- SimSWAP$GRParm2
tGRParm3 <- SimSWAP$GRParm3
tGRParm4 <- SimSWAP$GRParm4
tGRParm5 <- SimSWAP$GRParm5
tGRParm6 <- SimSWAP$GRParm6
# Number of separate studies (Figures)
NCorrStudy <- 6
# Loop over all correlation cases
#   Variable <- "CorFCLBCL"
#   Variable <- "CorFCSBCS"
#   Variable <- "CorFCCBCC"
#   Variable <- "CorFCLBCS"
#   Variable <- "CorFCSBCL"
#   Variable <- "CorFCSBCC"
#   << ADD MORE? >>
#
source('Swap MCVaR Core.R')
# Loop over number of correlations studied
for(kk in 1:NCorrStudy){
  MCRVaROutput <- tMCRVaROutput # Reset as file gets changed when outputting
  MCVaROutput <- tMCVaROutput # Reset as file gets changed when outputting
  InitialValuations <- tInitialValuations # Likely not necessary, but just in case
  if(kk == 1){
    Variable <- "CorFCLBCL"
  } else if(kk == 2){
    Variable <- "CorFCSBCS"
  } else if(kk == 3){
    Variable <- "CorFCCBCC"
  } else if(kk == 4){
    Variable <- "CorFCLBCS"
  } else if(kk == 5){
    Variable <- "CorFCSBCL"
  } else if(kk == 6){
    Variable <- "CorFCSBCC"
  } 
# Sensitivity Variable: NumberOfCorrelations in table not random variable related
  Increment <- (UpperBound - LowerBound)/(NumberOfCorrelations - 1)
  SensitivityVariable = data.frame(matrix(NA, nrow = NumberOfCorrelations, ncol = 1))
  colnames(SensitivityVariable)  <- 'Parameter'
  rownames(SensitivityVariable)  <- seq(1:NumberOfCorrelations)
  for(i in 1:NumberOfCorrelations){
    SensitivityVariable$Parameter[i] <- LowerBound + (i - 1)*Increment
  }
  Tab <- "Sensitivity Variable"
  SensitivityVariable <- cbind(Parameters = rownames(SensitivityVariable), SensitivityVariable)
  rownames(SensitivityVariable) <- 1:nrow(SensitivityVariable)
  wb <- buildWorkbook(SensitivityVariable, sheetName = Tab, gridLines = TRUE)
  BASECASE <- FALSE
  if(Variable == "CorFCLBCL"){ # Capture original values
    tCorFCLBCL <- MVNSimulationParameters$FDFCLevel[4]
  }
  if(Variable == "CorFCSBCS"){
    tCorFCSBCS <- MVNSimulationParameters$FDFCSlope[5]
  }
  if(Variable == "CorFCCBCC"){
    tCorFCCBCC <- MVNSimulationParameters$FDFCCurvature1[6]
  }
  if(Variable == "CorFCLBCS"){
    tCorFCLBCS <- MVNSimulationParameters$FDFCLevel[5]
  } 
  if(Variable == "CorFCSBCL"){
    tCorFCSBCL <- MVNSimulationParameters$FDFCSlope[4]
  } 
  if(Variable == "CorFCSBCC"){
    tCorFCSBCC <- MVNSimulationParameters$FDFCSlope[6]
  } 
  for(i in 1:NumberOfCorrelations){
    if(Variable == "CorFCLBCL"){ # Corr(FCLevel, BCLevel)
      MVNSimulationParameters$FDFCLevel[5] <- LowerBound + (i - 1)*Increment
      MVNSimulationParameters$FDBCLevel[1] <- MVNSimulationParameters$FDFCLevel[5]
    }
    if(Variable == "CorFCSBCS"){ # Corr(FCSlope, BCSlope)
      MVNSimulationParameters$FDFCSlope[4] <- LowerBound + (i - 1)*Increment
      MVNSimulationParameters$FDBCSlope[2] <- MVNSimulationParameters$FDFCSlope[4]
    }
    if(Variable == "CorFCSBCC"){ # Corr(FCSlope, BCCurvature1)
      MVNSimulationParameters$FDFCSlope[6] <- LowerBound + (i - 1)*Increment
      MVNSimulationParameters$FDBCCurvature1[2] <- MVNSimulationParameters$FDFCSlope[6]
    }
    if(Variable == "CorFCLBCS"){ # Corr(FCLevel, BCSlope)
      MVNSimulationParameters$FDFCLevel[5] <- LowerBound + (i - 1)*Increment
      MVNSimulationParameters$FDBCSlope[1] <- MVNSimulationParameters$FDFCLevel[5]
    }
    if(Variable == "CorFCSBCL"){ # Corr(FCSlope, BCLevel)
      MVNSimulationParameters$FDFCSlope[4] <- LowerBound + (i - 1)*Increment
      MVNSimulationParameters$FDBCLevel[2] <- MVNSimulationParameters$FDFCSlope[4]
    }
    if(Variable == "CorFCSBCC"){ # Corr(FCSlope, BCCurvature1)
      MVNSimulationParameters$FDFCSlope[6] <- LowerBound + (i - 1)*Increment
      MVNSimulationParameters$FDBCCurvature1[2] <- MVNSimulationParameters$FDFCSlope[6]
    }
    tMCOutput <- fSWAPMCVaRCore(SimSWAP, MVNSimulationParameters, InitialValuations, Variable)
    MCVaROutput[,i] <- tMCOutput$VaR
    MCRVaROutput[,i] <- tMCOutput$RVaR
  }
# Restore to original
  if(Variable == "CorFCLBCL"){
    MVNSimulationParameters$FDFCLevel[4] <- tCorFCLBCL
    MVNSimulationParameters$FDBCLevel[1] <- MVNSimulationParameters$FDFCLevel[4]
  }
  if(Variable == "CorFCSBCS"){
    MVNSimulationParameters$FDFCSlope[5] <- tCorFCSBCS
    MVNSimulationParameters$FDBCSlope[2] <- MVNSimulationParameters$FDFCSlope[5]
  }
  if(Variable == "CorFCSCCC"){
    MVNSimulationParameters$FDFCCurvature1[6] <- tCorFCCBCC
    MVNSimulationParameters$FDBCCurvature1[3] <- MVNSimulationParameters$FDFCCurvature1[6]
  }
  
  if(Variable == "CorFCLBCS"){ # Corr(FCLevel, BCSlope)
    MVNSimulationParameters$FDFCLevel[5] <- tCorFCLBCS
    MVNSimulationParameters$FDBCSlope[1] <- MVNSimulationParameters$FDFCLevel[5]
  }
  if(Variable == "CorFCSBCL"){ # Corr(FCSlope, BCLevel)
    MVNSimulationParameters$FDFCSlope[4] <- tCorFCSBCL
    MVNSimulationParameters$FDBCLevel[2] <- MVNSimulationParameters$FDFCSlope[4]
  }
  if(Variable == "CorFCSBCC"){ # Corr(FCSlope, BCCurvature1)
    MVNSimulationParameters$FDFCSlope[6] <- tCorFCSBCC
    MVNSimulationParameters$FDBCCurvature1[2] <- MVNSimulationParameters$FDFCSlope[6]
  }
  # VaR output based on sensitivities
  MCVaROutput <- cbind(Strategy = rownames(MCVaROutput), MCVaROutput)
  rownames(MCVaROutput) <- 1:nrow(MCVaROutput)
  Tab <- "MCVaR Output"
  addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
  writeData(wb, Tab, MCVaROutput)
  # Return VaR output based on sensitivities
  MCRVaROutput <- cbind(Strategy = rownames(MCRVaROutput), MCRVaROutput)
  rownames(MCRVaROutput) <- 1:nrow(MCRVaROutput)
  Tab <- "MCRVaR Output"
  addWorksheet(wb, sheetName = Tab, gridLines = TRUE)
  writeData(wb, Tab, MCRVaROutput)
  FilePath <- "Selected Output/"
  WorkBookName <- paste0(FilePath, 'Swap MC VaR and RVaR ', Variable, ' 1', '.xlsx')
  saveWorkbook(wb, WorkBookName, overwrite = TRUE)
  BASECASE <- TRUE
  Valuations <- fSWAPMCVaRCore(SimSWAP, MVNSimulationParameters, InitialValuations, Variable)
  rm(wb)
  tEndRunTime <- Sys.time()
  NumberOfSimulations
  tRunTime <- tEndRunTime - LagtEndRunTime
  LagtEndRunTime <- tEndRunTime
  print(tRunTime)
}
#
# Build volatility study
#  Volatility of each parameter: FC and BC; L, S, C
#


# Entire program
EndTime <- Sys.time()
NumberOfSimulations
RunTime <- EndTime - StartTime
RunTime
beep(sound = 2, print('Finished'))
